/**
 * @brief eeprom flash map api
 * @file eeprom_flash_map.c
 * @version 1.0
 * @author panzidong
 * @date 2018-9-27 10:59:53
 * @par Copyright:
 * Copyright (c) Anosi 2000-2018. All rights reserved.
 *
 * @history
 * 1.Date        : 2018-9-27 10:59:53
 *   Author      : panzidong
 *   Modification: Created file
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include "eeprom_flash_map.h"
/*----------------------------------------------*
 * external variables                           *
 *----------------------------------------------*/

/*----------------------------------------------*
 * external routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * internal routine prototypes                  *
 *----------------------------------------------*/

/*----------------------------------------------*
 * project-wide global variables                *
 *----------------------------------------------*/

/*----------------------------------------------*
 * module-wide global variables                 *
 *----------------------------------------------*/

/*----------------------------------------------*
 * constants                                    *
 *----------------------------------------------*/

/*----------------------------------------------*
 * macros                                       *
 *----------------------------------------------*/
#define DEVICE_FILE_KV3 "/sys/bus/i2c/devices/1-0050/eeprom"
#define DEVICE_FILE_KV4 "/sys/bus/i2c/devices/0-0050/eeprom"
/*if am335x use rtc chip pcf8563, eeprom access address will be change to follow*/
#define DEVICE_FILE_KV4_COMPAT_RTC_PCF8563 "/sys/bus/i2c/devices/0-0054/eeprom"
#define DEVICE_FILE_RK3328 "/sys/bus/i2c/devices/0-0050/eeprom"

#define GetKerverVersioCMD  "cat  /proc/version"
#define GetRtcChipNameCMD   "cat  /sys/class/rtc/rtc0/name"
#define KV3  "3.2.0"
#define KV4  "4.14.40"
#define KV_RK3328  "4.4.114"

/*----------------------------------------------*
 * routines' implementations                    *
 *----------------------------------------------*/

/**
 * @brief  ShowVersion
 *
 * @return none
 *
 * @history
 * 1.Date         : 2018-9-27 11:33:52
 *   Author       : panzidong
 *   Modification : Created function
 **/
void ShowVersion()
{
	printf ( "V5.0\n" );
}

/**
 * @brief display  eeprom tool usage
 *
 * @param [in] char* progname
 * @param [out] None
 *
 * @return none
 *
 * @history
 * 1.Date         : 2018-9-27 17:35:26
 *   Author       : panzidong
 *   Modification : Created function
 */
static void DisplayUsage ( char* progname )
{
	fprintf ( stderr, "Usage: %s\n" \
	          "  [-v | --version]....................................: Display all versions infomation\n" \
	          "  [--dump_magic]......................................: Dump magic info\n" \
	          "  [--write_magic string]..............................: Wirte magic info\n" \
	          "  [--dump_produce_time]...............................: Dump produce_time\n" \
	          "  [--write_produce_time string].......................: Wirte produce_time\n" \
	          "  [--dump_product_name]...............................: Dump product_name\n" \
	          "  [--write_product_name string].......................: Wirte product_name\n" \
	          "  [--dump_hardware_ver]...............................: Dump hardware_ver\n" \
	          "  [--write_hardware_ver string].......................: Wirte hardware_ver\n" \
	          "  [--dump_product_type]...............................: Dump product_type\n" \
	          "  [--write_product_type string].......................: Wirte product_type\n" \
	          "  [--dump_product_sn].................................: Dump product_sn\n" \
	          "  [--write_product_sn string].........................: Wirte product_sn\n" \
	          "  [--dump_product_key]................................: Dump product_key\n" \
	          "  [--write_product_key string]........................: Wirte product_key\n" \
	          "  [--dump_ali_product_key]............................: Dump ali_product_key\n" \
	          "  [--write_ali_product_key string]....................: Wirte ali_product_key\n" \
	          "  [--dump_ali_device_secret]..........................: Dump ali_device_secret\n" \
	          "  [--write_ali_device_secret string]..................: Wirte ali_device_secret\n" \
	          "  [-h | --help]..............................: Display this help\n" , progname );
}


/**
 * @brief get cmd resuslts
 *
 * @param [in] unsigned char* cmd
 * @param [in] unsigned char* result
 * @param [in] int size
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-11-26 15:20:27
 *   Author       : panzidong
 *   Modification : Created function
 */
int ExecuteCMD ( unsigned char* cmd, unsigned char* result, int size )
{
	FILE* fp = popen ( cmd, "r" );
	if ( NULL == fp ) {
		printf ( "popen error\n" );
		return -1;
	}

	int nbytes = fread ( result, sizeof ( unsigned char ), size, fp );
	if ( nbytes > 0 && nbytes < size - 1 )
		result[nbytes] = '\0';

	pclose ( fp );
	return nbytes;
}


/**
 * @brief get linux kernel version
 *
 * @param [in] unsigned char* result
 * @param [in] int size
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-11-26 15:22:9
 *   Author       : panzidong
 *   Modification : Created function
 */
int GetKernelVerion ( unsigned char* result, int size )
{
	int tmp = ExecuteCMD ( GetKerverVersioCMD, result, size );
	return tmp;
}

/**
 * @brief get rtc chip info
 *
 * @param [in] unsigned char* result
 * @param [in] int size
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2019-9-6 10:22:9
 *   Author       : panzidong
 *   Modification : Created function
 */
int GetRtcChipName ( unsigned char* result, int size )
{
        int tmp = ExecuteCMD ( GetRtcChipNameCMD, result, size );
        return tmp;
}

/**
 * @brief judge  kernel version is 3.2.0
 *
 * @param [in] None
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-11-26 15:22:40
 *   Author       : panzidong
 *   Modification : Created function
 */
int isKV3()
{
	char result[512] = {0};
	int size = 512;
	int tmp = 0;
	tmp = GetKernelVerion ( result, size );
	if ( tmp < 0 ) {
		return -1;
	}
	if ( strstr ( result, KV3 ) != NULL ) {
		return 0;
	}

	return -1;

}

/**
 * @brief judeg kernel version is 4.14.40
 *
 * @param [in] None
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-11-26 15:24:15
 *   Author       : panzidong
 *   Modification : Created function
 */
int isKV4()
{
	char result[512] = {0};
	int size = 512;
	int tmp = 0;
	tmp = GetKernelVerion ( result, size );
	if ( tmp < 0 ) {
		return -1;
	}
	if ( strstr ( result, KV4 ) != NULL ) {
		return 0;
	}

	return -1;

}


/**
 * @brief judeg kernel version is rk3328 4.4.114
 *
 * @param [in] None
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2019-4-8 15:24:15
 *   Author       : panzidong
 *   Modification : Created function
 */
int isKV_RK3328()
{
	char result[512] = {0};
	int size = 512;
	int tmp = 0;
	tmp = GetKernelVerion ( result, size );
	if ( tmp < 0 ) {
		return -1;
	}
	if ( strstr ( result, KV_RK3328 ) != NULL ) {
		return 0;
	}

	return -1;

}

/**
 * @brief judeg board use pcf8563 rtc chip
 *
 * @param [in] None
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2019-9-6 15:24:15
 *   Author       : panzidong
 *   Modification : Created function
 */
int isPcf8563RtcChip()
{
        char result[512] = {0};
        int size = 512;
        int tmp = 0;
        tmp = GetRtcChipName ( result, size );
        if ( tmp < 0 ) {
                return -1;
        }
        if ( strstr ( result, "rtc-pcf8563" ) != NULL ) {
                return 0;
        }

        return -1;

}


/**
 * @brief judeg board use rx8025 rtc chip
 *
 * @param [in] None
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2019-9-6 15:24:15
 *   Author       : panzidong
 *   Modification : Created function
 */
int isRx8025RtcChip()
{
        char result[512] = {0};
        int size = 512;
        int tmp = 0;
        tmp = GetRtcChipName ( result, size );
        if ( tmp < 0 ) {
                return -1;
        }
        if ( strstr ( result, "rtc-rx8025" ) != NULL ) {
                return 0;
        }

        return -1;

}


static int eeprom_fb = 0;


/**
 * @brief open file get fd
 *
 * @param [in] char* file_path
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 14:38:34
 *   Author       : panzidong
 *   Modification : Created function
 */
int eeprom_open ( char* file_path )
{
	int fd;
	fd = open ( file_path, O_RDWR );
	if ( fd < 0 ) {
		printf ( "Open %s fail\n", file_path );
		return -1;
	}
	return fd;
}

/**
 * @brief close file release fd
 *
 * @param [in] int fd
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 14:39:8
 *   Author       : panzidong
 *   Modification : Created function
 */
int eeprom_close ( int fd )
{
	if ( fd >  0 ) {
		close ( fd );
		return 0;
	} else {
		return -1;
	}
}
/**
 * @brief dump eeprom info
 *
 * @param [in] int fd
 * @param [in] int base
 * @param [in] int max_length
 * @param [out] buf
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 13:24:46
 *   Author       : panzidong
 *   Modification : Created function
 */
int eeprom_dump ( int fd, int base, int max_length, char* buf )
{

	char tmp_buf[256] = {0};
	int  result = 0 ;
	int  size = 0;
	int  i = 0;

	if ( fd < 0  || base < 0 )
		return -1;

	if ( ( base + max_length + 1 ) >  EEPROM_SIZE )
		return -1;

	lseek ( fd, base, SEEK_SET );

	result = read ( fd, tmp_buf, max_length + 1 );

	if ( result  < 1 )
		return  -1;

	size =  tmp_buf[0];

	if ( size  >  max_length || size  <  0 )
		return -1;

	if ( buf == NULL ) {
		tmp_buf[1 + size] = '\0';
		printf ( "base : %04x, length : %04d, context : %s\n", base, size, ( tmp_buf + 1 ) );
	} else {
		memcpy ( buf, tmp_buf + 1, size );
		return size;
	}
	return  0;
}

/**
 * @brief eeprom write
 *
 * @param [in] int fd
 * @param [in] int base
 * @param [in] int length
 * @param [in] char *buf
 *
 * @return int
 *
 * @history
 * 1.Date         : 2018-9-27 13:58:25
 *   Author       : panzidong
 *   Modification : Created function
 */
int eeprom_write ( int fd, int base, int length, char* buf )
{

	int  result = 0 ;
	char  size = ( char ) length;

	if ( fd < 0  || base < 0 || length < 0 || buf == NULL )
		return -1;

	if ( ( base + length + 1 ) >  EEPROM_SIZE )
		return -1;

	// write length
	lseek ( fd, base, SEEK_SET );
	result = write ( fd, &size, 1 );

	if ( result  != 1 )
		return  -1;

	// write body
	lseek ( fd, base + 1, SEEK_SET );
	result = write ( fd, buf, length );

	if ( result  != length )
		return  -1;

	return  0;

}


/**
 * @brief dump flash spec section context
 *
 * @param [in] int base
 * @param [in] int max_length
 * @param [in] char *buf
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 14:23:39
 *   Author       : panzidong
 *   Modification : Created function
 */
int am335x_eeprom_dump ( int base, int max_length, char* buf )
{
	int result;
	if ( isKV3() == 0 ){
		eeprom_fb = eeprom_open ( DEVICE_FILE_KV3 );
	}else if ( isKV4() == 0 ){
		if( isPcf8563RtcChip() == 0){
                        eeprom_fb = eeprom_open ( DEVICE_FILE_KV4_COMPAT_RTC_PCF8563 );
                }else{
                        eeprom_fb = eeprom_open ( DEVICE_FILE_KV4 );
                }
	}else if ( isKV_RK3328() == 0 ){
		eeprom_fb = eeprom_open ( DEVICE_FILE_RK3328 );
	}

	if ( eeprom_fb < 0 ) {
		return  -1;
	}

	if ( buf == NULL )
		result  = eeprom_dump ( eeprom_fb,  base,  max_length, NULL );
	else
		result  = eeprom_dump ( eeprom_fb,  base,  max_length, buf );

	eeprom_close ( eeprom_fb );

	return result;

}

/**
 * @brief write eeprom spec section context
 *
 * @param [in] int base
 * @param [in] int max_length
 * @param [in] char* buf
 * @param [out] None
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 14:22:44
 *   Author       : panzidong
 *   Modification : Created function
 */
int am335x_eeprom_write ( int base, int max_length, char* buf )
{
        int result;
        if ( isKV3() == 0 ){
                eeprom_fb = eeprom_open ( DEVICE_FILE_KV3 );
        }else if ( isKV4() == 0 ){
                if( isPcf8563RtcChip() == 0){
                        eeprom_fb = eeprom_open ( DEVICE_FILE_KV4_COMPAT_RTC_PCF8563 );
                }else{
                        eeprom_fb = eeprom_open ( DEVICE_FILE_KV4 );
                }
        }else if ( isKV_RK3328() == 0 ){
                eeprom_fb = eeprom_open ( DEVICE_FILE_RK3328 );
        }

	if ( eeprom_fb < 0 ) {
		return  -1;
	}

	system ( "echo 0  > /sys/class/leds/eeprom-write-control/brightness" );
	if ( strlen ( buf ) > max_length )
		result  = eeprom_write ( eeprom_fb, base, max_length, buf );
	else
		result  = eeprom_write ( eeprom_fb,  base, strlen ( buf ), buf );

	system ( "echo 255  > /sys/class/leds/eeprom-write-control/brightness" );
	eeprom_close ( eeprom_fb );
	return result;

}

/**
 * @brief parse parameter
 *
 * @param [in] int argc
 * @param [in] unsigned char*argv
 *
 * @return
 *
 * @history
 * 1.Date         : 2018-9-27 14:0:41
 *   Author       : panzidong
 *   Modification : Created function
 */
int  ParseOption ( int argc, char** argv )
{
	//Parse options
	while ( 1 ) {
		int option_index = 0, c = 0;
		static struct option log_options[] =  {
			{"h", no_argument, 0, 0},
			{"help", no_argument, 0, 0},

			{"v", no_argument, 0, 0},
			{"version", no_argument, 0, 0},

			{"dump_magic", no_argument, 0, 0},
			{"write_magic", required_argument, 0, 0},

			{"dump_produce_time", no_argument, 0, 0},
			{"write_produce_time", required_argument, 0, 0},

			{"dump_product_name", no_argument, 0, 0},
			{"write_product_name", required_argument, 0, 0},

			{"dump_hardware_ver", no_argument, 0, 0},
			{"write_hardware_ver", required_argument, 0, 0},

			{"dump_product_type", no_argument, 0, 0},
			{"write_product_type", required_argument, 0, 0},

			{"dump_product_sn", no_argument, 0, 0},
			{"write_product_sn", required_argument, 0, 0},

			{"dump_product_key", no_argument, 0, 0},
			{"write_product_key", required_argument, 0, 0},

			{"dump_ali_product_key", no_argument, 0, 0},
			{"write_ali_product_key", required_argument, 0, 0},
			
			{"dump_ali_device_secret", no_argument, 0, 0},
			{"write_ali_device_secret", required_argument, 0, 0},

			{0, 0, 0, 0}
		};
		c = getopt_long_only ( argc, argv, "", log_options, &option_index );

		if ( c == -1 ) {
			//No more options to parse
			break;
		}

		if ( c == '?' ) {
			//Unrecognized options
			DisplayUsage ( argv[0] );
			exit ( 0 );
		}

		switch ( option_index ) {
		//h, help
		case 0:
		case 1:
			DisplayUsage ( argv[0] );
			exit ( 0 );
			break;

		//v, version
		case 2:
		case 3:
			ShowVersion();
			exit ( 0 );
			break;

		//dump_magic
		case 4:
			//printf("dump_magic\n");
			am335x_eeprom_dump ( MAGIC_BASE, MAGIC_MAX_LENGTH, NULL );
			break;
		//write magic
		case 5:
			//printf("write_magic\n");
			am335x_eeprom_write ( MAGIC_BASE, MAGIC_MAX_LENGTH, optarg );
			break;

		//dump_pruduce_time
		case 6:
			//printf("dump_produce_time\n");
			am335x_eeprom_dump ( PRODUCE_TIME_BASE, PRODUCE_TIME_MAX_LENGTH, NULL );
			break;
		//write_produce_time
		case 7:
			//printf("write_produce_time\n");
			am335x_eeprom_write ( PRODUCE_TIME_BASE, PRODUCE_TIME_MAX_LENGTH, optarg );
			break;

		//dump_product_name
		case 8:
			am335x_eeprom_dump ( PRODUCT_NAME_BASE, PRODUCT_NAME_MAX_LENGTH, NULL );
			break;
		//write_product_name
		case 9:
			am335x_eeprom_write ( PRODUCT_NAME_BASE, PRODUCT_NAME_MAX_LENGTH, optarg );
			break;

		//dump_hardware_ver
		case 10:
			am335x_eeprom_dump ( HARDWARE_VER_BASE, HARDWARE_VER_MAX_LENGTH, NULL );
			break;
		//write_hardware_ver
		case 11:
			am335x_eeprom_write ( HARDWARE_VER_BASE, HARDWARE_VER_MAX_LENGTH, optarg );
			break;

		//dump_product_type
		case 12:
			am335x_eeprom_dump ( PRODUCT_TYPE_BASE, PRODUCT_TYPE_MAX_LENGTH, NULL );
			break;
		//write_product_type
		case 13:
			am335x_eeprom_write ( PRODUCT_TYPE_BASE, PRODUCT_TYPE_MAX_LENGTH, optarg );
			break;

		//dump_product_sn
		case 14:
			am335x_eeprom_dump ( PRODUCT_SN_BASE, PRODUCT_SN_MAX_LENGTH, NULL );
			break;
		//write_product_sn
		case 15:
			am335x_eeprom_write ( PRODUCT_SN_BASE, PRODUCT_SN_MAX_LENGTH, optarg );
			break;

		//dump_product_key
		case 16:
			am335x_eeprom_dump ( PRODUCT_KEY_BASE, PRODUCT_KEY_MAX_LENGTH, NULL );
			break;
		//write_product_key
		case 17:
			am335x_eeprom_write ( PRODUCT_KEY_BASE, PRODUCT_KEY_MAX_LENGTH, optarg );
			break;

		//dump_ali_product_key
		case 18:
			am335x_eeprom_dump ( ALI_PRODUCT_KEY_BASE, ALI_PRODUCT_KEY_MAX_LENGTH, NULL );
			break;
		//write_ali_product_key
		case 19:
			am335x_eeprom_write ( ALI_PRODUCT_KEY_BASE, ALI_PRODUCT_KEY_MAX_LENGTH, optarg );
			break;

		//dump_ali_device_secret
		case 20:
			am335x_eeprom_dump ( ALI_DEVICE_SECRET_BASE, ALI_DEVICE_SECRET_MAX_LENGTH, NULL );
			break;
		//write_ali_device_secret
		case 21:
			am335x_eeprom_write ( ALI_DEVICE_SECRET_BASE, ALI_DEVICE_SECRET_MAX_LENGTH, optarg );
			break;

		default :
			DisplayUsage ( argv[0] );
			break;
		}
	}

	return 0;
}

// int main ( int argc, char** argv )
// {
// 	ParseOption ( argc, argv );
// 	return 0;
// }
